#!/usr/bin/python
# -*- coding: utf-8 -*-

import logging
from os.path import expanduser

from cliff.command import Command

from fish_bundles.cli.lock import Lock


def update_init_file():
    lock = Lock.load()

    configurations = []

    for bundle in lock.bundles:
        cfg = lock.get_config_path(bundle)
        if cfg is not None:
            bundle_path = lock.get_bundle_path(bundle)
            configurations.append('    set -gx __fish_bundles_bundle_root_path %s' % bundle_path)
            configurations.append('    . %s' % cfg)

    with open(expanduser('~/.config/fish/functions/fish_bundles_init.fish'), 'w') as fish_init:
        fish_init.write('''
function fish_bundles_init --description="Initializes fish bundle paths and configurations"
    set -ex __fish_bundles_list
    . ~/.config/fish/bundles/functions_path.fish
%s
    set -ex __fish_bundles_bundle_root_path
end
'''.strip() % ("\n".join(configurations)))


class Init(Command):
    "Initializes fish-bundles."

    log = logging.getLogger(__name__)

    def take_action(self, parsed_args):
        print "Verifying fish configuration..."
        update_init_file()
        print "Fish configuration updated."



#!/usr/bin/python
# -*- coding: utf-8 -*-

import logging

from cliff.lister import Lister
from blessings import Terminal

from fish_bundles.cli.lock import Lock


class List(Lister):
    "Lists all installed bundles."

    log = logging.getLogger(__name__)

    def __init__(self, *args, **kw):
        super(List, self).__init__(*args, **kw)
        self.term = Terminal()

    def take_action(self, parsed_args):
        lock = Lock.load()
        result = []

        self.app.stdout.write(self.term.bold_blue(u'\nInstalled Bundles:\n'))
        for bundle in lock.bundles:
            author, repo = bundle['repo'].split('/')
            version = bundle['version']
            result.append((repo, version, author))

        return tuple((('bundle', 'version', 'author'),) + (result, ))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from json import dumps, loads
from os import makedirs, environ
from os.path import dirname, exists, expanduser, join

from semantic_version import Version


class Lock(object):
    def __init__(self, bundles):
        self.bundles = bundles

    @classmethod
    def get_default_path(cls):
        return environ.get('__fish_bundles_lock_file', expanduser('~/.config/fish/bundles.lock'))

    @classmethod
    def get_functions_path_file(cls):
        return environ.get('__fish_bundles_import_file', expanduser('~/.config/fish/bundles/functions_path.fish'))

    @property
    def bundles_path(self):
        return environ.get('__fish_bundles_root', expanduser('~/.config/fish/bundles'))

    def get_bundle_path(self, bundle):
        return join(self.bundles_path, bundle['repo'])

    def get_bundle_functions_path(self, bundle):
        return join(self.get_bundle_path(bundle), 'functions')

    def get_bundle_config_path(self, bundle):
        return join(self.get_bundle_path(bundle), 'config.fish')

    def get_config_path(self, bundle):
        config_path = self.get_bundle_config_path(bundle)
        if not exists(config_path):
            return None

        return config_path

    @classmethod
    def load(self, path=None):
        if path is None:
            path = Lock.get_default_path()

        if not exists(path):
            return Lock([])

        with open(path, 'r') as lock_file:
            content = lock_file.read()
            data = loads(content)
            return Lock(data)

    def save(self, path=None):
        if path is None:
            path = Lock.get_default_path()

        if not exists(dirname(path)):
            makedirs(dirname(path))

        data = dumps(self.bundles)
        with open(path, 'w') as lock_file:
            lock_file.write(data)

        self.update_function_paths()

    def update_function_paths(self):
        paths = []
        for bundle in self.bundles:
            functions_path = self.get_bundle_functions_path(bundle)
            paths.append("set -gx fish_function_path $fish_function_path %s\n" % functions_path)

        path = Lock.get_functions_path_file()
        with open(path, 'w') as import_file:
            for path in paths:
                import_file.write(path)

    def get_bundle(self, key):
        match = filter(lambda item: item['repo'] == key, self.bundles)
        if not match:
            return None

        return match[0]

    def reload(self):
        new_bundles = []
        for bundle in self.bundles:
            bundle_path = self.get_bundle_path(bundle)
            if exists(bundle_path):
                new_bundles.append(bundle)
        self.bundles = new_bundles

    def update_bundle(self, bundle):
        match = self.get_bundle(bundle['repo'])
        if match:
            self.bundles.remove(match)
        self.bundles.append(bundle)

    def is_up_to_date(self, bundle):
        key = bundle['repo']
        match = self.get_bundle(key)

        if not match:
            return False

        return Version(match['version']) >= Version(bundle['version'])



#!/usr/bin/python
# -*- coding: utf-8 -*-

import logging
import sys

from cliff.app import App
from cliff.commandmanager import CommandManager
import requests  # NOQA

from fish_bundles import __version__


class FishBundlesApp(App):

    log = logging.getLogger('fish-bundles')

    def __init__(self):
        super(FishBundlesApp, self).__init__(
            description='fish-bundles',
            version=__version__,
            command_manager=CommandManager('fb'),
        )


def main(argv=None):
    requests_log = logging.getLogger("requests")
    requests_log.setLevel(logging.WARNING)

    if argv is None:
        argv = sys.argv[1:]
    app = FishBundlesApp()
    return app.run(argv)


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



#!/usr/bin/python
# -*- coding: utf-8 -*-

import logging
from json import dumps
import sys
import shutil
from os import environ, makedirs
from os.path import join, dirname, exists, expanduser
import tempfile
from zipfile import ZipFile
from cStringIO import StringIO

from cliff.lister import Lister
import requests
from blessings import Terminal
from requests.exceptions import ConnectionError

from fish_bundles.cli.lock import Lock
from fish_bundles.cli.init import update_init_file


class Install(Lister):
    "Installs the configured bundles."

    log = logging.getLogger(__name__)

    def __init__(self, *args, **kw):
        super(Install, self).__init__(*args, **kw)
        self._term = None

    @property
    def term(self):
        if self._term is None:
            if self.options.boring:
                self._term = Terminal(force_styling=None)
            else:
                self._term = Terminal()

        return self._term

    def get_parser(self, prog_name):
        parser = super(Install, self).get_parser(prog_name)
        parser.add_argument('--force', action='store_true',  default=False)
        parser.add_argument('--boring', action='store_true',  default=False)
        return parser

    def get_dim(self, msg):
        return self.term.dim_yellow(msg)

    def get_error_message(self, message, details):
        return self.term.bold_red('\nError: %s\n\n%s%sError details: %s\n' % (
            message, self.term.normal, self.term.dim_white, details
        ))

    def show_warning(self, msg, extra_line=True):
        separator = "%s\n" % ('-' * len(msg))
        self.app.stdout.write(separator)
        self.app.stdout.write(self.term.bold_white_on_yellow(msg) + '\n')
        self.app.stdout.write(separator)
        if extra_line:
            self.app.stdout.write('\n')

    def take_action(self, parsed_args):
        self.options = parsed_args

        if '__fish_bundles_list' not in environ:
            self.show_warning(
                'Warning: Could not find the "__fish_bundles_list" environment variable. '
                'Have you added any \'fish_bundle "bundle-name"\' entries in your config.fish file?\n'
            )

        self.ensure_user_token()

        bundles = environ.get('__fish_bundles_list', '')
        bundles = list(set(bundles.split(':')))
        bundles = ['fish-bundles/root-bundle-fish-bundle'] + [bundle for bundle in bundles if bundle]
        server = environ.get('__fish_bundles_host', 'http://bundles.fish/')
        bundle_path = environ.get('__fish_bundles_root', expanduser('~/.config/fish/bundles'))

        info = self.get_bundle_info(server, bundles)
        installed, lock = self.install(info, bundle_path, parsed_args.force)

        if not installed:
            self.app.stdout.write(self.term.bold_green(
                u'\n☺ YAY! All bundles are up-to-date. No new bundles were installed. ☺ \n'
            ))
        else:
            installed_bundles = ", ".join(
                [bundle[1] for bundle in installed]
            )
            self.app.stdout.write(self.term.bold_green(
                '\nSuccessfully installed %d bundles (%s)!\n\n%s%sUpdated Bundle Versions:\n' % (
                    len(installed),
                    installed_bundles,
                    self.term.normal,
                    self.term.bold_blue
                )
            ))

            update_init_file()

        result = []

        self.app.stdout.write(self.term.bold_blue(u'\nInstalled Bundles:\n'))
        for bundle in lock.bundles:
            author, repo = bundle['repo'].split('/')
            version = bundle['version']
            result.append((repo, version, author))

        return tuple((('bundle', 'version', 'author'),) + (result, ))

    def install(self, info, bundle_path, force):
        tmp_dir = tempfile.mkdtemp()
        installed_bundles = []
        lock = Lock.load()

        for bundle in info:
            if not force and lock.is_up_to_date(bundle):
                self.copy_bundle(bundle, to=tmp_dir)
                continue

            logging.info(self.get_dim('>>> Installing %s...' % bundle['repo']))
            self.unzip(bundle, to=tmp_dir)
            author, repo = bundle['repo'].split('/')
            logging.info(self.get_dim('>>> %s installed successfully.' % bundle['repo']))
            installed_bundles.append((author, repo, bundle['version']))
            lock.update_bundle(bundle)

        shutil.rmtree(bundle_path)
        shutil.copytree(tmp_dir, bundle_path)
        lock.reload()
        lock.save()

        return installed_bundles, lock

    def copy_bundle(self, bundle, to):
        bundle_path = environ.get('__fish_bundles_root', expanduser('~/.config/fish/bundles'))
        bundle_dir = join(bundle_path.rstrip('/'), bundle['repo'].lstrip('/'))
        to_dir = join(to.rstrip('/'), bundle['repo'].lstrip('/'))
        shutil.copytree(bundle_dir, to_dir)

    def is_allowed_path(self, path):
        return path.endswith('.fish')

    def unzip(self, bundle, to):
        data = requests.get(bundle['zip'])
        z = ZipFile(StringIO(data.content))

        files = z.filelist

        root = files[0].filename

        root_path = join(to.rstrip('/'), bundle['repo'].lstrip('/'))

        for zip_file in files[1:]:
            path = zip_file.filename.replace(root, '').lstrip('/')
            if not self.is_allowed_path(path):
                continue

            file_path = join(root_path.rstrip('/'), path)
            file_dir = dirname(file_path)
            if not exists(file_dir):
                makedirs(file_dir)

            with open(file_path, 'w') as writer:
                with z.open(zip_file) as reader:
                    writer.write(reader.read())

    def get_bundle_info(self, server, bundles):
        try:
            result = requests.get("%s/my-bundles" % server.rstrip('/'), params=dict(bundles=dumps(bundles)))
        except ConnectionError:
            err = sys.exc_info()[1]
            raise RuntimeError(self.get_error_message(
                'Could not process the bundles. fish-bundles server was not found or an error happened.',
                str(err)
            ))

        if result.status_code != 200:
            raise RuntimeError(self.get_error_message(
                'Could not process the bundles. fish-bundles server was not found or an error happened.',
                'status code - %s' % result.status_code
            ))

        data = result.json()

        if data['result'] != 'bundles-found':
            raise RuntimeError(self.get_error_message(
                'Could not process the bundles. %s' % data['error'],
                'status code - %s\n' % result.status_code
            ))

        return data['bundles']

    def ensure_user_token(self):
        token_path = environ.get('__fish_bundles_token_path', expanduser('~/.fbrc'))
        if not exists(token_path):
            # https://help.github.com/articles/creating-an-access-token-for-command-line-use
            self.show_warning(
                "We still can't find your authentication tokens for github. Your connectivity may be limited.",
                extra_line=False
            )
            self.app.stdout.write("==> Please verify the docs on how to configure it at http://github.com/whateverurl.\n\n")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of fish-bundles.
# https://github.com/fish-bundles/fb

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2014 Bernardo Heynemann heynemann@gmail.com


import os
from setuptools import setup, find_packages
from setuptools.command.install import install as _install
from fish_bundles import __version__


class install(_install):
    def run(self):
        _install.run(self)
        os.system('fb init')
        try:
            os.system('fb install --boring')
        except Exception:
            print "-------------------------------------------------------------"
            print "An error happened while trying to install the latest bundles."
            print "You **MUST** run 'fb install' after this finishes."
            print "-------------------------------------------------------------"


tests_require = [
    'mock',
    'nose',
    'coverage',
    'yanc',
    'preggy',
    'tox',
    'ipdb',
    'coveralls',
    'sphinx',
]

setup(
    cmdclass={'install': install},
    name='fish-bundles',
    version=__version__,
    description='fb is the command line tool for fish-bundles',
    long_description='''
fb is the command line tool for fish-bundles.
''',
    keywords='fish fish-bundles cli shell',
    author='Bernardo Heynemann',
    author_email='heynemann@gmail.com',
    url='https://github.com/fish-bundles/fb',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: Unix',
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: Implementation :: PyPy",
        'Operating System :: OS Independent',
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'cliff==1.6.1',
        'blessings==1.5.1',
        'semantic_version==2.3.0',
    ],
    extras_require={
        'tests': tests_require,
    },
    entry_points={
        'console_scripts': [
            # add cli scripts here in this form:
            'fb=fish_bundles.cli:main',
        ],
        'fb': [
            'install = fish_bundles.cli.install:Install',
            'init = fish_bundles.cli.init:Init',
            'list = fish_bundles.cli.list:List',
        ],
    },
)
